
/*
 * File:   delays.c
 * Author: sprut
 *
 * Created on 3. Mrz 2020
 * 
 * alles was mit dem LCD zu tun hat
 * 
 * char / byte
 * 
 */


#include "mytypes.h"
#include "signalgen.h"
#include "lcd.h"

//*****************************************************************************************************************
// LCD
//*****************************************************************************************************************

// Enable=1
void LCDE_1(void)
{
    LCD_E = 1;
	__delay_us(100); //SleepUs(100);
}

// Enable=0
void LCDE_0(void)
{
	LCD_E = 0;	
	__delay_us(100); //SleepUs(100);
}

// warten bis das display bereit ist
void LCD_BUSY(void)
{
	BYTE puffer;
	//RD4..7 auf input
	TRISD |= 0xF0;
	LCD_RW = 1;
	LCD_RS = 0;
	__delay_us(100); //SleepUs(100);
	do
	{
		LCDE_1();
		puffer._byte = PORTD;
		LCDE_0();
		LCDE_1();
		LCDE_0();
	} while (puffer.b7==1);
	//RD4..7 auf output
	TRISD &= 0x0F;
	LCD_RW = 0;
}


//8 Bit als 2x4 Bit zum LCD schreiben
void LCD_Data(byte daten)
{
	//high
	LATD = (LATD & 0x0F) | (daten & 0xF0);
	LCDE_1();
	LCDE_0();
	//low
	LATD = (LATD & 0x0F) | ((daten & 0x0F) << 4);
	LCDE_1();
	LCDE_0();
}

// zeichen schreiben
void LCD_Out(byte daten)
{
	LCD_BUSY();
	LCD_RS = 1;
	LCD_Data(daten);
}

//steuerung
void LCD_Control(byte daten)
{
	LCD_BUSY();
	LCD_RS = 0;
	LCD_Data(daten);
}

//steuerung
void LCD_Control_noBF(byte daten)
{
	LCD_RS = 0;
	LCD_Data(daten);
}


void LCD_Init(void){
	// einschalten,  2-zeilig
    __delay_ms(20); 
    LCDE_0();
    LCD_D4 = 0;
    LCD_D5 = 0;
    LCD_D6 = 0;
    LCD_D7 = 0;
    LCD_RS = 0;
    LCD_RW = 0;

    LCD_D4 = 1;		// 8 bit
    LCD_D5 = 1;
    __delay_us(100); 
    LCDE_1();
    LCDE_0();
    __delay_ms(20); // 0011****

    LCDE_1();		// 8 bit
    LCDE_0();
    __delay_us(100); // 0011**** //  100+100us

    LCDE_1();		// 8 bit
    LCDE_0();
    __delay_us(100); // 0011**** // 100+100us

    LCD_D4 = 0;		// 4 bit
    LCD_D5 = 1;
    __delay_us(100); 
    LCDE_1();
    LCDE_0();		// 0010****

    LCD_Control_noBF(0x28);     __delay_us(100);  // 4 bit,  2-zeilig
    LCD_Control_noBF(0x08);		__delay_us(100);  //display off
    LCD_Control_noBF(0x01);		__delay_ms(10);   //lschen
    LCD_Control_noBF(0x06);		__delay_us(100);  //entry mode, increment, disable display-shift
    //LCD_Control_noBF(0x0F);	SleepUs(100);	  //on, kursor on, blink	//display on
    //LCD_Control_noBF(0x0C);		__delay_us(100);  //on, kursor off, no-blink	//display on
    LCD_Control_noBF(0x0E);     __delay_us(100);  //on, kursor on, no-blink
}    


void LCD_Cursor_On(void){
    //LCD_Control(0x0F);          //on, kursor on, blink
    LCD_Control(0x0E);          //on, kursor on, no-blink
}


void LCD_Cursor_Off(void){
    LCD_Control(0x0C);          //on, kursor off, no-blink
}


void LCD_Pos(char zeile, char spalte) {
    // Kursor auf zeile [2] Spalte [3]
	// 2: Zeile 0..3
	// 3: Spalte 0..39
	// Zeile 0: 0x00
	// Zeile 1: 0x40
	// Zeile 2: 0x00+Zeilenlnge
	// Zeile 3: 0x40+Zeilenlnge
	if (zeile > 1) {spalte += 20;}  //Zeile2, 3 		
	byte Adresse = ((zeile & 0x01) * 0x40) + spalte;
	LCD_Control(Adresse | 0x80);
}


void LCD_String(char *S, char length) {
    char c;
    for (c=0; c<length; c++)
    {
        if (*S==0) return;
       LCD_Out(*S);
       S++;
    }
}




//*****************************************************************************************************************
/*
   4 321.654 987 MHz
  0.603 mW  -2.2 dBm
  INT            OFF
  Frq  MHz  kHz Menu
 */


/*
    4 321.654 987 MHz
  0.603 mW  -2.2 dBm
  INT            OFF
  Pow  dBm   mW Menu
 */

/*
    4 321.654 987 MHz
  0.603 mW  -2.2 dBm
  INT            OFF
  Save  P+   P-  Back
 */