/*
 * File:   newmain.c
 * Author: sprut
 *
 * Created on 8. September 2019, 18:26
 */


/*
 * Pinout
 * RA0 < KEY (147*)
 * RA1 < KEY (2580)
 * RA2 < KEY (369#)
 * RA3 < KEY (ABCD)
 * RA4 - KEY (<>s.)
 * RA5 < ADF-MUX
 * RA6 > OSC2
 *   
 * RB0 < SPI_SDI   int0
 * RB1 > SPI_SCK   int1
 * RB2 -           int2  < KEY via diodes
 * RB3 -
 * RB4 <      ipc   < RotEnc A
 * RB5 < PGM  ipc   < RotEnc B
 * RB6 < PGC  ipc   < RotEnc Push
 * RB7 < PGD  ipc 
 *   
 * RC0 > KEY (123A)
 * RC1 > KEY (456B)
 * RC2 > KEY (789C)
 * xxx
 * RC4 - D-
 * RC5 - D+
 * RC6 > TX-RS232
 * RC7 < RX > SPI_SDO
 * 
 **RD0 -
 **RD1 > LCD_RS
 **RD2 > LCD_RW
 **RD3 > LCD_E
 **RD4 - LCD_D0/4
 **RD5 - LCD_D1/5
 **RD6 - LCD_D2/6
 **RD7 - LCD_D3/7
 ** 
 **RE0 > ADF_CS
 **RE1 > ATT_CS
 **RE2 > KEY (*0#D)
 * RE3 < MCLR
 *
 * 
 */

/*
 * signed char          8       Signed integer
 * unsigned char        8       Unsigned integer 
 * signed short         16      Signed integer
 * unsigned short       16      Unsigned integer
 * signed int           16      Signed integer
 * unsigned int         16      Unsigned integer
 * __int24              24      Signed integer
 * __uint24             24      Unsigned integer
 * signed long          32      Signed integer
 * unsigned long        32      Unsigned integer
 * signed long long     32/64   Signed integer
 * unsigned long long   32/64   Unsigned integ
 * 
 * The __bit and __int24 types are non-standard types available in this implementa-tion. 
 * The long long types are 64-bit C99 standard types when building for PIC18 or 
 * Enhanced Mid-range devices, but this implementation limits their size to only 32 bits 
 * for projects conforming to the C90 Standard or any project targeting any other device.
 * 
 * All integer values are represented in little endian format with the Least Significant 
 * Byte (LSB) at the lower address.If no signedness is specified in the type, then the 
 * type will be signed except for the char and __bit types which are always unsigned. 
 * The concept of a signed bit is meaningless.
 * 
 * Signed values are stored as a two?s complement integer value.
 *
 */

//*****************************************************************************************************************
// PIC18F4550 Configuration Bit Settings **************************************************************************
//*****************************************************************************************************************

// 'C' source line config statements

// CONFIG1L
#pragma config PLLDIV = 5       // PLL Prescaler Selection bits (Divide by 5 (20 MHz oscillator input))
#pragma config CPUDIV = OSC3_PLL4// System Clock Postscaler Selection bits ([Primary Oscillator Src: /3][96 MHz PLL Src: /4])
#pragma config USBDIV = 2       // USB Clock Selection bit (used in Full-Speed USB mode only; UCFG:FSEN = 1) (USB clock source comes from the 96 MHz PLL divided by 2)

// CONFIG1H
#pragma config FOSC = HSPLL_HS  // Oscillator Selection bits (HS oscillator, PLL enabled (HSPLL))
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor disabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRT = ON        // Power-up Timer Enable bit (PWRT enabled)
#pragma config BOR = OFF        // Brown-out Reset Enable bits (Brown-out Reset disabled in hardware and software)
#pragma config BORV = 3         // Brown-out Reset Voltage bits (Minimum setting 2.05V)
#pragma config VREGEN = ON      // USB Voltage Regulator Enable bit (USB voltage regulator enabled)

// CONFIG2H
#pragma config WDT = OFF        // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = ON      // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = OFF     // PORTB A/D Enable bit (PORTB<4:0> pins are configured as digital I/O on Reset)
#pragma config LPT1OSC = OFF    // Low-Power Timer 1 Oscillator Enable bit (Timer1 configured for higher power operation)
#pragma config MCLRE = ON       // MCLR Pin Enable bit (MCLR pin enabled; RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF        // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config ICPRT = OFF      // Dedicated In-Circuit Debug/Programming Port (ICPORT) Enable bit (ICPORT disabled)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 (000800-001FFFh) is not code-protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 (002000-003FFFh) is not code-protected)
#pragma config CP2 = OFF        // Code Protection bit (Block 2 (004000-005FFFh) is not code-protected)
#pragma config CP3 = OFF        // Code Protection bit (Block 3 (006000-007FFFh) is not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) is not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM is not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (000800-001FFFh) is not write-protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (002000-003FFFh) is not write-protected)
#pragma config WRT2 = OFF       // Write Protection bit (Block 2 (004000-005FFFh) is not write-protected)
#pragma config WRT3 = OFF       // Write Protection bit (Block 3 (006000-007FFFh) is not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) are not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot block (000000-0007FFh) is not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM is not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (000800-001FFFh) is not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (002000-003FFFh) is not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection bit (Block 2 (004000-005FFFh) is not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection bit (Block 3 (006000-007FFFh) is not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot block (000000-0007FFh) is not protected from table reads executed in other blocks)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

//******************************************************************************************************

#include <xc.h>
#include <stdio.h>      
#include <math.h>  
#include <string.h>
#include <stdint.h>

#include "signalgen.h"
#include "mytypes.h"
#include "lcd.h"
#include "spi.h"

 
//******************************************************************************************************
// EEPROM-Data
//******************************************************************************************************

__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x00 - 0x07
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x08 - 0x0F
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x10 - 0x17
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x18 - 0x1F
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x40,0x78,0x7D,0x01); // 0x20 - 0x27  25000000 Hz
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x28 - 0x2F
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x30 - 0x37
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x38 - 0x3F
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x40 - 0x47
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x48 - 0x4F
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00); // 0x50 - 0x57  R = 1
__EEPROM_DATA(0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00); // 0x58 - 0x5F

#define RS232


#define E_Adr_PowerCal      0x00    // 20 byte bis 0x23
#define E_Adr_Quarz_Hz_I    0x24    //  4 byte bis 0x27
#define E_Adr_PowerCal2     0x28    // 46 byte bis 0x55 
#define E_Adr_R_W           0x56    //  2 byte bis 0x57

//****************************************************************************************************************
// variables
//****************************************************************************************************************
unsigned char sync_mode;
unsigned char bus_mode;
unsigned char smp_phase;

char        key;
char        Rotor;
char        Rot_inc;
char        Rot_dec;
signed char Rot_Delta;
word        R_Word;

float       Adf_Frq;
int         Adf_Frq_MHz;
int         Adf_Frq_kHz;
signed long Frq_Delta_kHz;
signed long Quarz_Hz_I;
signed long Frq_Delta_Quarz;
float       Adf_Frq_Set;         // am ADF eingestellte Frequenz
char        Adf_PowerLevel;
float       Adf_PowerLevel_Set;  // am ADF eingestellte Power

signed int  Out_cBm;
signed int  Pow_Delta_cBm;
char        ADF_Lock;
char        ADF_Fail;
char        ATT_Fail;
char        INT_Flag;
char        ON_Flag;
char        MUX_State;
char        Rot_Pushed;
char        State;
char        KeyBuf[20];


//    0 ...  250 -> 0
//  250 ...  750 -> 1
//  750 ... 1250 -> 3
//
// 4250 ... 4750 -> 9
// Gainkorrekturtabellen in 0,1 dB
// Frequenz [GHz]              0, 0.5,   1, 1.5,   2, 2.5,   3, 3.5,   4, 4.5    - 10 values [0..9]
signed int Amp_Gain[10]  = { 210, 206, 202, 200, 195, 187, 195, 180, 174, 170}; // +20dB; gain in 0,1dB-increments
signed int Att_Loss[10]  = { -17, -15, -18, -22, -21, -20, -20, -22, -23, -24}; //  -2dB; loss in 0,1 dB increments
signed int Adf_Loss[10]  = { -55, -40, -35, -31, -20, -10,  -3, -25, -40, -40}; //  -4dB; loss (compared to default) in 0,1dB increments
signed int Correction[10]= {  -6,   2,   4,   5,   0,   2,  14, -15,  -3, -18}; // larger value -> more power
signed int PowerCal[10]  = {   0,   0,   0,   0,   0,   0,   0,   0,   0,   0}; // larger value -> more power

// Frequenz [GHz]              0, 0.2, 0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6, 1.8,
signed int PowerCal2[24] = {   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,  // larger value -> more power
// Frequenz [GHz]            2.0, 2.2, 2.4, 2.6, 2.8, 3.0, 3.2, 3.4, 3.6, 3.8,
                               0,   0,   0,   0,   0,   0,   0,   0,   0,   0,
// Frequenz [GHz]            4.0, 4.2, 4.4
                               0,   0,   0};

// 0 GHz  1.2  dB  1.5 dB
// 1 GHz  1.4  dB  1.8 dB
// 2 GHz  1.6  dB  2.0 dB
// 3 GHz  1.75 dB  2.0 dB
// 4 GHz  2.0  dB  2.3 dB

//****************************************************************************************************************
// vorabdeklarationen
//****************************************************************************************************************

void TestAdf(void);
float SetFrq(int Freq_MHz, int Freq_kHz);
void KeyBoardPressed(void);
void RotorTurned(void);
void Init(void);
void clearBufLine(void);
void Menuline(void);
void Lcd_Welcome(void);
char f2f(void);
char f2f200(void);
signed int interpolate(signed int *buffer);
signed int interpolate200(signed int *buffer);
void SetAtt(unsigned char ATT);
signed int Get_AttLoss_cB(void);
void SetFrqDbm(void);
void cBm2Lcd(signed int cbm);
void Freq2Lcd(float freq, char Line);
void Int2Lcd(int value, char length, char line, char pos);
void Freq2Lcd_I(signed long freq_Hz, char line);
void Cursor_Care(void);
void SaveFrqData(void);
void SaveCalData(void);
void SaveCal2Data(void);
void ReadCalData(void);
void ReadCal2Data(void);
void LoadFrqData(void);
word ReadEepromWord(char adr);
void WriteEepromWord(char adr, word data);
byte MyEeprom_read(byte addr);
void MyEeprom_write(byte addr, byte n);

//****************************************************************************************************************
/** I N T E R R U P T S  *******************************************/
//****************************************************************************************************************
//#pragma interruptlow InterruptHandler
//void InterruptHandler (void) {


void __interrupt (high_priority) my_isr_routine (void) { // 0x0008

    unsigned char pufferA;    
    unsigned char pufferB;
     
    // das ist der Interrupt fr keyboard 
    if (INTCON3bits.INT2IF) {
        
     // tastatur abfragen 
        // RA0 < KEY
        // RA1 < KEY
        // RA2 < KEY
        // RA3 < KEY
        // 1 2 3 A  RC0
        // 4 5 6 B  RC1
        // 7 8 9 C  RC2
        // * 0 # D  RE2
      key = 0xFF;
      LATC = LATC | 0x07;
      LATEbits.LATE2 = 1;
      
      LATCbits.LATC0 = 0;
      __delay_ms(1);
      pufferA = (PORTA & 0x0F);  // FF, FE, FD, FB, F7    1 2 3 A <         
      switch( pufferA ) {
        case 0x0E:                      
            key = '1';        
             break;    
        case 0x0D:                      
            key = '2';    
            break;
        case 0x0B:                       
            key = '3'; 
            break;
        case 0x07:                       
            key = 'A';    
            break;
        default:                      
            break;
      }                  
      LATCbits.LATC0 = 1;
      
      LATCbits.LATC1 = 0;
      __delay_ms(1);
      pufferA = (PORTA & 0x0F);  // FF, FE, FD, FB, F7     4 5 6 B >         
      switch( pufferA ) {
        case 0x0E:                      
            key = '4';        
             break;   
        case 0x0D:                       
            key = '5';    
            break;
        case 0x0B:                       
            key = '6'; 
            break;
        case 0x07:                       
            key = 'B';    
            break;
        default:                      
            break;
      }             
      LATCbits.LATC1 = 1;
      
      LATCbits.LATC2 = 0;
      __delay_ms(1);
      pufferA = (PORTA & 0x0F);  // FF, FE, FD, FB, F7        7 8 9 C Push      
      switch( pufferA ) {
        case 0x0E:                       
            key = '7';        
             break;    
        case 0x0D:                       
            key = '8';    
            break;
        case 0x0B:                      
            key = '9';    
            break;
        case 0x07:                     
            key = 'C';    
            break;
        default:                    
            break;
      }           
      LATCbits.LATC2 = 1;
      
      LATEbits.LATE2 = 0;
      __delay_ms(1);
      pufferA = (PORTA & 0x0F);  // FF, FE, FD, FB, F7              
      switch( pufferA ) {
        case 0x0E:                       
            key = '*';        
             break;    
        case 0x0D:                       
            key = '0';    
            break;
        case 0x0B:                      
            key = '#';    
            break;
        case 0x07:                     
            key = 'D';    
            break;
        default:                    
            break;
      }         
              
      LATC = LATC & 0xF8;
      LATEbits.LATE2 = 0;                
//      TxCharRs232(key);  
      INTCON3bits.INT2IF = 0; 
    }
    
    
    // das ist der Interrupt fr Rotary encoder
    if (INTCONbits.RBIF) {
      //* RB4 < KEY       ipc   < RotEnc A
      //* RB5 < KEY  PGM  ipc   < RotEnc B
      //* RB6 < KEY  PGC  ipc   < RotEnc Push
      //* RB7 < KEY  PGD  ipc 
 
      pufferB = PORTB; // RB4-5 ist A-B
      char newRotor = (pufferB >> 4) & 3;
      // 0>1>3>2>0
      if (((Rotor==0) && (newRotor==1)) || ((Rotor==1) && (newRotor==3)) || 
          ((Rotor==3) && (newRotor==2)) || ((Rotor==2) && (newRotor==0)))  Rot_inc=1; else Rot_inc=0; 
      if (((Rotor==0) && (newRotor==2)) || ((Rotor==2) && (newRotor==3)) || 
          ((Rotor==3) && (newRotor==1)) || ((Rotor==1) && (newRotor==0)))  Rot_dec=1; else Rot_dec=0; 
      
      if (Rot_inc) Rot_Delta++;
      if (Rot_dec) Rot_Delta--;
      
      if (pufferB & 0x40) Rot_Pushed=0; else Rot_Pushed=1;
      
      Rotor  = newRotor;  
      pufferB = PORTB;
      INTCONbits.RBIF = 0;      
    }
    
    
    if (TMR0IE && TMR0IF) {

        TMR0IF = 0;
    }    
}  //Interrupt



//*****************************************************************************************************************
// code
//*****************************************************************************************************************

void main(void) {
    
 
    
//    Int2Lcd(123, 4, 1, 0); //int value, char length, char line, char pos);
    
    char        ST[20];  
    int         k;
    unsigned    char pufferA;
    char        freecount;   
    char        new_ADF_Lock;
    char        str1[2];
    
    Quarz_Hz_I = 25000000;
    State      = State_FRQ;
    
//    Adf_Frq_MHz = 1300;
//    unsigned int test = -Get_AttLoss_cB();
//    test++;
    
    /*
    Adf_Frq_MHz =1000;
    Adf_Frq_kHz =    0;
    Adf_PowerLevel = 1;
    R_Word = 1;
    SetFrq(Adf_Frq_MHz, Adf_Frq_kHz);          
    */
    
    Init();  
    
   // TestAdf();
   
    Lcd_Welcome();
    
    #ifdef RS232
    OpenMyRs232();  
    TxCharMyRs232(0x55);  
    TxCharMyRs232(0x0A);  
    TxCharMyRs232(0x0D);  
    #endif
    
    // Frequenzen 100MHz ... 4.4GHz    
    /*
    di();
    Out_cBm = 0;
    for (k=500; k<4500; k=k+500) {
        Adf_Frq_MHz = k;
        Adf_Frq_kHz = 0;
        SetFrqDbm();
        __delay_ms(1000);
    }
    ei();              
*/
    
    Menuline();
    Cursor_Care();  
    
    ON_Flag     = 0;
    Out_cBm     = 0;
    Adf_Frq_MHz = 1000;
    Adf_Frq_kHz = 0;
    SetFrqDbm();   
    clearBufLine();
    Cursor_Care();

    while(1) {
        //keyboard
        if (key != 0xFF) {
            KeyBoardPressed();

            //warten auf key-not-pressed for 20ms
            freecount = 0;
            do {
                  pufferA = (PORTA & 0x0F);
                  if (pufferA==0x0F) freecount++;
                  else freecount=0;
                  __delay_ms(1);
            } while (freecount<20);

            key = 0xFF;        
        }
        //rot-encoder
        if(Rot_inc) {
            RotorTurned();
            Rot_inc = 0;
       } 
        if(Rot_dec) {
            RotorTurned();
            Rot_dec = 0;
        }
        //MUX
        new_ADF_Lock = ADF_MUX;
        if (new_ADF_Lock != ADF_Lock) {            
            LCD_Pos(0,1);
            if (ADF_Lock) strcpy(str1, " ");
            else          strcpy(str1, "?");
            LCD_String(str1, 1);                   
            ADF_Lock = new_ADF_Lock; 
            Cursor_Care();
        }
    }   
    return;
}   //main


// HF-Ausgang ein- aus-schalten
void SwitchOutput(void){
   if (ON_Flag) ON_Flag = 0;
           else ON_Flag = 1;
   SetFrqDbm();
}


// lschen der Eingabezeile im Display
void clearBufLine(void){
    LCD_Pos(2,5);
    char wipe[12] = "           ";  // 11 stellen
    char lenght = strlen(wipe); 
    LCD_String(wipe, lenght);
    
    // bei power-calibration den correction-value anzeigen
    signed int cal;
    char buffer[12];
    if (State==State_PCal) {
        //cal = interpolate(PowerCal);
        cal = interpolate200(PowerCal2);
        sprintf(buffer, "%i", cal);
        LCD_Pos(2,5);
        LCD_String(buffer, strlen(buffer));         
    } else
        
    // bei Quarz-calibration die Quarzfrequenz anzeigen
    if (State==State_FCal) {
        Freq2Lcd_I(Quarz_Hz_I, 2);
    } else
    
    // bei DEB R anzeigen
    if (State==State_DEB) {
        //Freq2Lcd_I(R_Word, 2);
        Int2Lcd(R_Word, 4, 2, 10); //int value, char length, char line, char pos);
    }
} // clearBufLine


void RotorTurned(void){
    signed long number;
    if (State == State_POW) {  // dBm
        if (Rot_Pushed) {
            if (Rot_inc) Pow_Delta_cBm = 1;
            if (Rot_dec) Pow_Delta_cBm = 10;
            Rot_Delta = 0;
        } else {  
             if (Rot_Delta >  2) {
                Out_cBm = Out_cBm + Pow_Delta_cBm;
                Rot_Delta = 0;
            }
            if (Rot_Delta < -2) {
                if (Rot_dec) Out_cBm = Out_cBm - Pow_Delta_cBm;
                Rot_Delta = 0;
            }                                   
            SetFrqDbm();
        }          
    } else 
    if (State == State_FRQ) {   // Frequenz
        if (Rot_Pushed) {
            if (Rot_Delta >  2) {
                if (Frq_Delta_kHz > 1) Frq_Delta_kHz /= 10; 
                Rot_Delta = 0;
            }
            if (Rot_Delta < -2) {
                if (Frq_Delta_kHz < 1000000) Frq_Delta_kHz *= 10; 
                Rot_Delta = 0;
            }                         
        } else {     
            // limit number
            number = Adf_Frq_MHz;
            number = (number *1000) + Adf_Frq_kHz;

            if (Rot_Delta >  2) {
                number += Frq_Delta_kHz; 
                Rot_Delta = 0;
                if (number > 4400000) number = 4400000;
                Adf_Frq_kHz = number % 1000;
                Adf_Frq_MHz = number / 1000;              
                SetFrqDbm();
            }
            if (Rot_Delta < -2) {
                number -= Frq_Delta_kHz; 
                Rot_Delta = 0;
                if (number <   35000) number =   35000;
                Adf_Frq_kHz = number % 1000;
                Adf_Frq_MHz = number / 1000;              
                SetFrqDbm();
            }                                
        }
    } else 
    if (State == State_FCal) {   // Quarz-Frequenz) 
        if (Rot_Pushed) {
            if (Rot_Delta >  2) {
                if (Frq_Delta_Quarz > 1) Frq_Delta_Quarz /= 10; 
                Rot_Delta = 0;
            }
            if (Rot_Delta < -2) {
                if (Frq_Delta_Quarz < 1000000) Frq_Delta_Quarz *= 10; 
                Rot_Delta = 0;
            }    
        } else {
            if (Rot_Delta >  2) {
                Quarz_Hz_I += Frq_Delta_Quarz; 
                Rot_Delta = 0;
                if (Quarz_Hz_I > 101000000) Quarz_Hz_I = 101000000;           
                SetFrqDbm();
            }
            if (Rot_Delta < -2) {
                Quarz_Hz_I -= Frq_Delta_Quarz; 
                Rot_Delta = 0;
                if (Quarz_Hz_I <   9000000) Quarz_Hz_I = 9000000;
                SetFrqDbm();
            }                                
        }                                    
    }    
    KeyBuf[0] = 0;
    clearBufLine();   
    Cursor_Care();
} //RotorTurned


// Tastendruck verarbeiten
void KeyBoardPressed(void){
    char lenght;  
    signed long number;
    
    switch (key) {
    case '#':   // RF ON  OFF
        SwitchOutput();
        break;
    case '<':
        if (Frq_Delta_kHz < 1000000) Frq_Delta_kHz *= 10;
        break; 
    case '>':
        if (Frq_Delta_kHz > 1) Frq_Delta_kHz /= 10;    
        break;    
    case '*':   // erase last digit
        // cut off last char
        lenght = strlen(KeyBuf);
        if (lenght>0) KeyBuf[lenght-1] = 0;
        clearBufLine();
        LCD_Pos(2,5);
        lenght = strlen(KeyBuf); 
        LCD_String(KeyBuf, lenght);
        break;        
    case '0': 
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':   
        if ((State == State_POW) || (State == State_FRQ)) {                    
            lenght = strlen(KeyBuf); // prfen auf lnge !!! frq:7  pow:2
            if ((lenght<2) || ((lenght<7) && (State == State_FRQ)) || ((lenght<4) && (State == State_DEB)) ) {
                strncat(KeyBuf, &key, 1); 
                LCD_Pos(2,5);
                lenght = strlen(KeyBuf); 
                LCD_String(KeyBuf, lenght);
            }
        }
        break;
    case 'A':   // Frq - Power - Umschaltung;  SAVE
        if (State == State_POW)  State = State_FRQ; else
        if (State == State_FRQ)  State = State_POW; else
        if (State == State_PCal) {  // save
            //SaveCalData();
            SaveCal2Data();
            LCD_Cursor_On();
            State = State_POW;
        } else
        if (State == State_FCal) {  // save
            SaveFrqData();
            LCD_Cursor_On();
            State = State_FRQ;
        } else
        if (State == State_DEB) {            
            WriteEepromWord(E_Adr_R_W, R_Word);
        }   
        Menuline();
        KeyBuf[0] = 0;
        clearBufLine();
        SetFrqDbm();    // eigentlich will ich nur die obersten beiden Zeilen wieder herstellen
        break;
    case 'B':   // MHz  dBm
        // read out buffer and convert in number
        number = atol(KeyBuf);        
        if (State == State_POW) {  // +dBm
            // limit number
            if (number < -30) number = -30;
            if (number >  15) number = +15;
            //set power
            Out_cBm = number*10;
        } else 
        if (State == State_FRQ) {   // MHz
            // limit number
            if (number <   35) number =   35;
            if (number > 4400) number = 4400;
            //set frequency
            Adf_Frq_MHz = number;
            Adf_Frq_kHz = 0;     
        } else
        if (State == State_PCal) {      //P+
            //PowerCal[f2f()]++;
            PowerCal2[f2f200()]++;
        } else
        if (State == State_FCal) {      //MHz+
            Quarz_Hz_I += 1000000;
            if (Quarz_Hz_I > 101000000) Quarz_Hz_I = 101000000;           
            SetFrqDbm();            
        } else
        if (State == State_DEB) {      //R+
            R_Word++;            
            if (R_Word > 1023) R_Word = 1023;                       
        }
        SetFrqDbm();
        KeyBuf[0] = 0;
        clearBufLine();
        break;    
    case 'C':   // kHz -dBm
        // read out buffer and convert in number
        number = atol(KeyBuf);        
        if (State == State_POW) {  // -dBm
            // limit number
            number = -number;
            if (number < -30) number = -30;
            if (number >  15) number = +15;
            //set power
            Out_cBm = number*10;
        } else         
        if (State == State_FRQ) {   // kHz
            //limit number 
            if (number <   35000) number =   35000;
            if (number > 4400000) number = 4400000;
            //set frequency
            Adf_Frq_kHz = number % 1000;
            Adf_Frq_MHz = number / 1000;
        } else
        if (State == State_PCal) {      //P-
            PowerCal[f2f()]--; 
            PowerCal2[f2f200()]--; 
        } else
        if (State == State_FCal) {      //Mhz-
            Quarz_Hz_I -= 1000000;
            if (Quarz_Hz_I <   9000000) Quarz_Hz_I = 9000000;       
            SetFrqDbm();                  
        } else
        if (State == State_DEB) {      //R-
            R_Word--;            
            if (R_Word == 0) R_Word = 1;                       
        } 
        SetFrqDbm();
        KeyBuf[0] = 0;
        clearBufLine();
        break;
    case 'D':  
        if (State == State_POW) {    // Menu
            State = State_PCal;
            LCD_Cursor_Off();
            Menuline();
            KeyBuf[0] = 0;
            clearBufLine();                       
        } else        
        if (State == State_FRQ) {    // Menu
            State = State_FCal;
            Menuline();
            KeyBuf[0] = 0;
            clearBufLine();                       
        } else
        if (State == State_PCal) {    // Back
            State = State_POW;
            LCD_Cursor_On();
            Menuline();
            KeyBuf[0] = 0;
            clearBufLine();
        } else
        if (State == State_FCal) {    // Back
            State = State_FRQ;
            Menuline();
            KeyBuf[0] = 0;
            clearBufLine();
        } else
        if (State == State_DEB) {    // Back
            State = State_FRQ;
            Menuline();
            KeyBuf[0] = 0;
            clearBufLine();
        }
        clearBufLine();
        break;
    //default:
    //    anweisung30;
    }
    __delay_ms(10); // entprellen
    key = 0xFF; 
    Cursor_Care();
}   //KeyBoardPressed


// setzt bei Bedarf den blinkenden Kursor 
void Cursor_Care(void){
    char Spalte;
    if (State == State_POW) {  // -cBm
        LCD_Cursor_On();
        if (Pow_Delta_cBm == 1) Spalte =  5; else                
                                Spalte =  3;                    
        LCD_Pos(1, Spalte);                        
    } else         
    if (State == State_FRQ) {   // kHz
        LCD_Cursor_On();
        if (Frq_Delta_kHz <      2) Spalte = 11; else
        if (Frq_Delta_kHz <     20) Spalte = 10; else
        if (Frq_Delta_kHz <    200) Spalte =  9; else

        if (Frq_Delta_kHz <   2000) Spalte =  7; else
        if (Frq_Delta_kHz <  20000) Spalte =  6; else            
        if (Frq_Delta_kHz < 200000) Spalte =  5; else                
                                    Spalte =  3;                    
        LCD_Pos(0, Spalte);
    } else
    if (State == State_FCal) {   // kHz        
        LCD_Cursor_On();
        if (Frq_Delta_Quarz <         2) Spalte = 15; else
        if (Frq_Delta_Quarz <        20) Spalte = 14; else
        if (Frq_Delta_Quarz <       200) Spalte = 13; else
                    
        if (Frq_Delta_Quarz <      2000) Spalte = 11; else
        if (Frq_Delta_Quarz <     20000) Spalte = 10; else
        if (Frq_Delta_Quarz <    200000) Spalte =  9; else
            
        if (Frq_Delta_Quarz <   2000000) Spalte =  7; else
                                         Spalte =  6;
        LCD_Pos(2, Spalte);
    }
} //Cursor_Care


// beschreibe unterste Zeile mit Tastenbelegung fr A...D
void Menuline(void){
    LCD_Pos(3,0);
    char str1[21];
    char lenght;
    if (State==State_FRQ)  strcpy(str1, " Pow  MHz  kHz  QCal"); else
    if (State==State_POW)  strcpy(str1, " Frq  dBm -dBm  PCal"); else
    if (State==State_PCal) strcpy(str1, " Save  P+   P-  Back"); else
    if (State==State_FCal) strcpy(str1, " Save +MHz -MHz Back"); else
    if (State==State_DEB)  strcpy(str1, " Save  R+   R-  Back");
    lenght = strlen(str1); 
    LCD_String(str1, lenght);    
}


void Lcd_Welcome(void) {
    char str1[21];
    char lenght;
    LCD_Pos(0,0); strcpy(str1, "  Signalgenerator"); lenght = strlen(str1); LCD_String(str1, lenght);   
    LCD_Pos(1,0); strcpy(str1, "   35Mhz - 4.4GHz"); lenght = strlen(str1); LCD_String(str1, lenght);   
    LCD_Pos(2,0); strcpy(str1, "  -20dBm - +15dBm"); lenght = strlen(str1); LCD_String(str1, lenght);   
    LCD_Pos(3,0); strcpy(str1, " V01  by sprut.de"); lenght = strlen(str1); LCD_String(str1, lenght);   
    __delay_ms(3000);
    LCD_Control(0x01);
    __delay_ms(10);   //lschen   
}


// initialisiert die Hardware un Variablen
void Init(void) {
    char puffer;
    
    Frq_Delta_Quarz= 1000;     // 1 MHz
    Frq_Delta_kHz  = 1000;     // 1 MHz
    Adf_Frq_MHz    = 1000;     // 1 GHz
    Adf_Frq_kHz    = 0;
    Adf_PowerLevel = 1;        // -1dBm
    Out_cBm        = 0;        // 0 dBm
    Pow_Delta_cBm  = 10;       // 1 dBm per step
    ADF_Lock       = 0;
    INT_Flag       = 0;
    ON_Flag        = 1;
    MUX_State      = 0;
    ADF_Fail       = 0;
    ATT_Fail       = 0;
    Rot_Pushed     = 0;
    KeyBuf[0]      = 0;
	sync_mode      = SPI_FOSC_16;		//Master, Fosc/16
	bus_mode       = SPI_MODE_00;
	smp_phase      = SMPEND;            // Input data sample at end of data out           
    OpenSPI(sync_mode,bus_mode,smp_phase);				// init SPI-Hardware und setzt TRIS-Bits
    ReadCalData();
    ReadCal2Data();
    
    // Quarzfrequenz
    LoadFrqData();
    if ((Quarz_Hz_I < 9000000) || (Quarz_Hz_I > 101000000)) {
        Quarz_Hz_I = 25000000;
        SaveFrqData();
    }
    
    //Division ratios from 1 to 1023 are allowed
    R_Word = ReadEepromWord(E_Adr_R_W);
    if (R_Word>1023) R_Word = 5;
    if (R_Word == 0) R_Word = 5;
      
    // init LCD  alles an PortD
    TRISD = 0x01;       
    LATD  = 0xFF;   
    LCD_Init(); 
    
    // init SPI-pins
    PORTE  = 0;
    LATE   = 0;
    ADCON1 = 0x0F;  // alle AN sind digital pins  
    CMCON  = 7;     // comperator off    
    ADF_CS = 1;
    ATT_CS = 1;
    TRISEbits.TRISE0 = 0;       // ADF_CS-output
    TRISEbits.TRISE1 = 0;       // ATT_CS-output    
    
    // init Tastatur   RC0,1,2,RE2 (output)   RA0..3 input
    key = 0xFF;
    TRISCbits.TRISC0 = 0;       // define RC0 as output
    TRISCbits.TRISC1 = 0;       // define RC1 as output
    TRISCbits.TRISC2 = 0;       // define RC2 as output
    TRISEbits.TRISE2 = 0;       // define RE2 as output
    LATC = LATC & 0xF8;
    LATEbits.LATE2   = 0;
    
    // interrupts
    RCONbits.IPEN    = 0;       // keine high and low priority interrupts ist immer 0x08
    
    // enable int-on change an RB(4..7)
    INTCON2bits.RBPU = 0;         // pull up
    __delay_ms(10); 
    if (PORTBbits.RB6 == 0) State = State_DEB;
    else                    State = State_FRQ;
        
    puffer           = PORTB;
    INTCONbits.RBIF  = 0;         // flag loeschen
    INTCONbits.RBIE  = 1;         // enable
    INTCON2bits.RBIP = 1;         // int on 008h
    INTCONbits.GIEH  = 1;    
    
    // enable int an INT2 (RB2) 
    INTCON3bits.INT2IF  = 0;      // flag loeschen
    INTCON2bits.INTEDG2 = 0;      // int2 on falling edge
    INTCON3bits.INT2IE  = 1;      // enable
    INTCON3bits.INT2IP  = 1;      // high priority 008h
    INTCONbits.GIEH     = 1;   
      
    //init Rotary encoder
    Rotor     = (PORTB >> 4) & 3;
    Rot_inc   = 0;
    Rot_dec   = 0;
    Rot_Delta = 0;
    
    ei();       // enable interrupts
} //Init


void TestAdf(void){
    DWORD           adf[6];
    signed char     k; 
    
    
    LCD_Out('0');
    adf[0]._dword = 0x00320000;
    adf[1]._dword = 0x08008009;
    adf[2]._dword = 0x00004E42; 
    adf[3]._dword = 0x008004B3;
    adf[4]._dword = 0x008C803C;
    adf[5]._dword = 0x00580005; 
    
    // SPI###########################################################
    //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000); 

    LCD_Out('0');
//    adf[4]._dword = 0x008FA024;
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000); 
    LCD_Out('0');
    adf[4]._dword = 0x008C803C;
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000); 
    
    LCD_Out('2');
    adf[2]._dword = 0x18004E42; 
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000);     
    LCD_Out('0');
    adf[2]._dword = 0x00004E42; 
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000);    
    
    LCD_Out('1');
    adf[1]._dword = 0x0800CE21; 
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000);  
    LCD_Out('0');
    adf[1]._dword = 0x08008009; 
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000);  
    
    
    LCD_Out('4');
    adf[4]._dword = 0x008FA024;
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000); 
    LCD_Out('0');
    adf[4]._dword = 0x008C803C;
   //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000);    
    
    
    LCD_Out('x');
    adf[0]._dword = 0x00320000;
    adf[1]._dword = 0x08008009;
    adf[2]._dword = 0x00004E42; 
    adf[3]._dword = 0x008004B3;
    adf[4]._dword = 0x008C803C;
    adf[5]._dword = 0x00580005; 
    
    // SPI###########################################################
    //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }  
    __delay_ms(3000);     
    
}


// input  Sollfrequenz in MHz
// output Istfrequenz in MHz
float SetFrq(int Freq_MHz, int Freq_kHz) {
    // wort 2
    #define Ref_Doubler         0x02000000
    #define Ref_Divider         0x01000000
    #define mux_lock_analog     0x14000000
    #define mux_lock_digital    0x18000000
    #define ldpin_lock_digital  0x00400000

    float           Quarz_MHz;
    signed char     k;  
    char            P_SET;
    char            try;
    DWORD           adf[6];
    float           Freq_F;
    unsigned int    Freq_Mhz;
    char            DIV_SET;
    unsigned int    RF_DIV;
    float           T, D, R, Step_MHz_f, INT_f, FRAC_f, Fout;
    unsigned int    INT, MODO, FRAC;
    unsigned long   BS_Clock_Divider;
    
    if (Freq_MHz > 4400) Freq_MHz = 4400;
    if (Freq_MHz < 35)   Freq_MHz = 35;         // 35 * 64 = 2240      34.375=2,2GHz/64
    if (Freq_kHz > 999)  Freq_kHz = 999;
    if (Freq_kHz < 0)    Freq_kHz = 0;         
     
    P_SET    = Adf_PowerLevel; // 1 = -1dBm    
    
    Freq_F   = Freq_kHz;
    Freq_F   = Freq_F / 1000 + Freq_MHz;  // frequency in MHz as float
    Freq_Mhz = trunc(Freq_F + 0.5);
    
    // for less than 2.2 GHz we need a divider DIV_SET
    // DIV_SET=0 ->  1/1
    // DIV_SET=6 -> 1/64
    DIV_SET = 0;
    RF_DIV  = 1;
    while (Freq_MHz < 2200) {
        DIV_SET++;
        RF_DIV   = RF_DIV   * 2;
        Freq_MHz = Freq_MHz * 2;    // VCO-freq in MHz in front of divider
        Freq_kHz = Freq_kHz * 2;
        if (Freq_kHz>999) {
            Freq_kHz = Freq_kHz - 1000;
            Freq_MHz++;
        }        
    }
    
    // now lets determine the lock-frequency for the PLL
    // 10 MHz = PFD
    Quarz_MHz  = Quarz_Hz_I;
    Quarz_MHz  = Quarz_MHz / 1000000;
    T          = 0;  // Ref_Divider
    D          = 0;  // Ref_Doubler
    R          = R_Word;                //1    
    Step_MHz_f = ((1+D)/(R*(1+T))) * Quarz_MHz;    //float Step_MHz_f = 25 
     
    // and now the dividers for the PLL
    INT_f    = (Freq_F / Step_MHz_f) * RF_DIV;
    INT      = trunc(INT_f);
    MODO     = 2500;    //4000;
    FRAC_f   = round( (INT_f - INT) * MODO);    
    FRAC     = trunc(FRAC_f);
    INT_Flag = (FRAC == 0);
    Fout     = Step_MHz_f * ( (float)INT + ((float)FRAC / (float)MODO)) / (float)RF_DIV;
    
    BS_Clock_Divider = Step_MHz_f * 10;  // 100 kHz
    
    
    // Array belegen ################################################
    //finally put everything into the adf-array for SPI
    adf[0]._dword = ((unsigned long int)INT  << 15) | ((unsigned long)FRAC << 3);
    adf[1]._dword = 0x00008001 | (MODO << 3);                                       // phase=1
    adf[2]._dword = 0x00000E42 | mux_lock_digital | ((unsigned long)R << 14) | ((unsigned long)T << 24) | ((unsigned long)D << 25); 
    adf[3]._dword = 0x00000003 | (150 << 3); // DB23
    adf[4]._dword = 0x00800004 | ((unsigned long)DIV_SET << 20) | (BS_Clock_Divider << 12);
    adf[5]._dword = 0x00180005 | ldpin_lock_digital;
        
    // falls FRAC zufllig 0 sein sollte, lsst sich noch was optimieren
    /*
    if (FRAC==0) {
        adf[2].Byte1.b0 = 1;  // D08   LDF                                        // spart 26 zellen
        adf[3].Byte2.b5 = 1;  // D21   charge cancelation
        adf[3].Byte2.b6 = 1;  // D22   ANTIBACKLASH PULSE WIDTH from 6 to 3 ns    // spart 24 zellen
    }
    */
    
    // bei bedarf 8/9=on
    if ((Freq_MHz>3600) || (INT>32768)) {
        adf[1].Byte3.b3 = 1;        // D27  PRESCALER 8/9                          // spart 26 zellen
    }

    // P_SET ist die Ausgangsleistungsstufe von 0=-4dBm; 1=-1dBm; 2=+2dBm; 3=+5dBm;
    adf[4]._dword = adf[4]._dword | ((unsigned long)P_SET << 3);
    
    // Power on/off
    if (ON_Flag) {
        adf[4].Byte0.b5 = 1;     // D05                                         // spart 16 zellen
    } 
    
    adf[2].Byte0.b5 = 0; //failsave , no power down
    adf[1].Byte3.b3 = 1;        // D27  PRESCALER 8/9              
    
    #ifdef RS232
    // RS232###########################################################
    for (k=5; k>=0; k--) {
        TxDWordMyRs232(adf[k]._dword);
        TxCharMyRs232(0x0A);  
        TxCharMyRs232(0x0D);
    }    
    #endif  
    
    // SPI###########################################################
    //send it to the ADF
    for (k=5; k>=0; k--) {
        ADF_CS   = 0;
        ADF_Fail = SPI32Tx(adf[k]._dword);
        ADF_CS   = 1;
    }
    /*
    ADF_CS   = 0;
    ADF_Fail = SPI32Tx(adf[0]._dword);
    ADF_CS   = 1;
    __delay_ms(20);
    
    try = 0;
    while ((ADF_MUX==0) && (try < 5)) {
        for (k=5; k>=0; k--) {
            ADF_CS   = 0;
            ADF_Fail = SPI32Tx(adf[k]._dword);
            ADF_CS   = 1;
        }
        try++;
        __delay_ms(20);
    }                      
     */  
    
    Adf_Frq_Set        = Fout;
    //Adf_PowerLevel_Set = P_SET;
    return(Fout);
} //SetFrq


// just writes the input value directly into the attenuator
// 0 .. 31.75 dB additional attenuation
void SetAtt(unsigned char ATT) {
    ATT_CS = 0;
    ATT_Fail = SPI8Tx(ATT);
    ATT_CS = 1;
    ATT_CS = 0;
}

//    0 ...  250 -> 0
//  250 ...  750 -> 1
//  750 ... 1250 -> 3
//
// 4250 ... 4750 -> 9
char f2f(void){
    signed char f = (Adf_Frq_MHz + 250) / 500;
    if (f<0) f = 0;
    if (f>9) f = 9;
    return f;
}

//    0 ...  100 -> 0
//  100 ...  300 -> 1
//  300 ...  500 -> 3
//
// 4300 ... 4500 -> 22
char f2f200(void) {
    signed char f = (Adf_Frq_MHz + 100) / 200;
    if (f<0 ) f = 0;
    if (f>22) f = 22;
    return f;   
}

signed int interpolate(signed int *buffer) {
    signed int p1, p2, fMitte, df, out;
    char f;
    float out_f;

    // Mittelfrequenz des bandes
    fMitte = ((Adf_Frq_MHz + 250) / 500) * 500;
    if (abs(Adf_Frq_MHz - fMitte)<50) return buffer[f2f()]; // +- 50 MHz keine Interpolation
    
    //bandnummer
    f = f2f();  // 0 ... 9
    if (Adf_Frq_MHz > fMitte) {
        if (f>8) return buffer[f];  // >> 4GHz
        p1 = buffer[f];
        p2 = buffer[f+1];
    } else {
        if (f<1) return buffer[f];  // <<500 MHz
        p1 = buffer[f-1];
        p2 = buffer[f];        
    }
    df    = Adf_Frq_MHz % 500;
    out_f = p2-p1;
    out_f = out_f*df;
    out_f = out_f/500;
    out_f = out_f + p1;
    //out = p1 + (p2-p1)*df/500;
    out = round(out_f);
    return out;
}

signed int interpolate200(signed int *buffer) {
    signed int p1, p2, fMitte, df, out;
    char f;
    float out_f;

    //bandnummer
    f = f2f200();   // 0 ... 22
    
    // Mittelfrequenz des bandes
    fMitte = ((Adf_Frq_MHz + 100) / 200) * 200;
    if (abs(Adf_Frq_MHz - fMitte)<10) return buffer[f]; // +- 10 MHz keine Interpolation   

    if (Adf_Frq_MHz > fMitte) {
        if (f>21) return buffer[f];  // >> 4,3GHz
        p1 = buffer[f];
        p2 = buffer[f+1];
    } else {
        if (f<1) return buffer[f];  // <<100 MHz
        p1 = buffer[f-1];
        p2 = buffer[f];        
    }
    df    = Adf_Frq_MHz % 200;
    out_f = p2-p1;
    out_f = out_f*df;
    out_f = out_f/200;
    out_f = out_f + p1;
    //out = p1 + (p2-p1)*df/200;
    out = round(out_f);
    return out;
} // interpolate200

// input  0,25 dB : 1 = 0.25 dB
signed int SetAtt_qdB(signed int Att_qdB){
    int AttNum = -Att_qdB;
    if (AttNum < 0  ) AttNum = 0;
    if (AttNum > 127) AttNum = 127;
    if (ON_Flag) SetAtt(AttNum); else SetAtt(127);
    return (-AttNum);
}

//frequenzabhngiger Gain des PA in dB/10
// use Adf_Frq_MHz
signed int Get_AmpGain_cB(void){   
    return interpolate(Amp_Gain);
}

//frequenzabhngiger Insertion-loss des Att in dB/10
// insertion loss:
// Frq   Datasheet gemessen
// 0 GHz  1.2  dB  1.5 dB
// 1 GHz  1.4  dB  1.8 dB
// 2 GHz  1.6  dB  2.0 dB
// 3 GHz  1.75 dB  2.0 dB
// 4 GHz  2.0  dB  2.3 dB
// use Adf_Frq_MHz
signed int Get_AttLoss_cB(void) {    
    return interpolate(Att_Loss);
}

//frequenzabhngiger Verluste AdF in dB/10 (Abweichung vom Soll)
// use Adf_Frq_MHz
signed int Get_AdfLoss_cB(void) {
    return interpolate(Adf_Loss);
}

//frequenzabhngiger Korrektur der Leistung dB/10 (Abweichung vom Soll)
// use Adf_Frq_MHz
signed int Get_Correction_cB(void) {
    return interpolate(Correction);
}

signed int Get_PowerCal_cB(void){
    //return interpolate(PowerCal);
    return interpolate200(PowerCal2);
}


void SetFrqDbm(void){
    char str1[5];
    char lenght;
    
    // limit fr Sollfrequenz
    if (Adf_Frq_MHz > 4400) Adf_Frq_MHz = 4400;
    if (Adf_Frq_MHz < 35)   Adf_Frq_MHz = 35;
    
    // limit fr power
    if (Out_cBm >  150) Out_cBm =  150;
    if (Out_cBm < -200) Out_cBm = -200;
    
    signed int BaseGain_cB  = Get_AmpGain_cB() + Get_AttLoss_cB() + Get_AdfLoss_cB() - Get_Correction_cB() - Get_PowerCal_cB();
    signed int MinPower_cBm = Out_cBm - BaseGain_cB;          // Output des ADF
    if (MinPower_cBm >  20) Adf_PowerLevel = 3;  else  // +5 dBm
    if (MinPower_cBm > -10) Adf_PowerLevel = 2;  else  // +2 dBm
    if (MinPower_cBm > -40) Adf_PowerLevel = 1;  else  // -1 dBm
                            Adf_PowerLevel = 0;        // -4 dBm
    signed int Adf_Soll_cBm = -40 + (Adf_PowerLevel * 30);
    signed int Att_qdB      = (Out_cBm - Adf_Soll_cBm - BaseGain_cB) * 2 / 5;
    
    LCD_Cursor_Off();
    cBm2Lcd( SetAtt_qdB(Att_qdB)*5/2 + Adf_Soll_cBm + BaseGain_cB);
    Freq2Lcd( SetFrq(Adf_Frq_MHz, Adf_Frq_kHz), 0);
    
    LCD_Pos(2,17);
    if (ON_Flag) strcpy(str1, "ON ");
    else         strcpy(str1, "OFF");
    lenght = strlen(str1); 
    LCD_String(str1, lenght);
    
    LCD_Pos(2,0);
    if (INT_Flag) strcpy(str1, "INT ");
    else          strcpy(str1, "FRAC");
    lenght = strlen(str1); 
    LCD_String(str1, lenght);     
    
    LCD_Pos(0,1);
    if (ADF_Lock) strcpy(str1, " ");
    else          strcpy(str1, "?");
    lenght = strlen(str1); 
    LCD_String(str1, lenght);
    
    LCD_Pos(0,0);
    if (ADF_Fail) strcpy(str1, "x");
    else          strcpy(str1, " ");
    lenght = strlen(str1); 
    LCD_String(str1, lenght);
    
    LCD_Pos(1,0);
    if (ATT_Fail) strcpy(str1, "x");
    else          strcpy(str1, " ");
    lenght = strlen(str1); 
    LCD_String(str1, lenght);
} //SetFrqDbm


// Anzeige der Ausgangsleistung in der Zeile 1
void cBm2Lcd(signed int cBm){
    LCD_Pos(1,1);
    char buffer[12];
    signed int dbmx = cBm;
    dbmx = dbmx * 10;       //    * 0,1 * 10
    sprintf(buffer, "%0+4.4i  dBm", dbmx); // "+12,34 dBm"
    
    if (buffer[1] == '0') buffer[1] = ' ';
    buffer[5] = buffer[4];
    buffer[4] = buffer[3];
    buffer[3] = '.';
    LCD_String(buffer, strlen(buffer)); 

    // +15 dBm = 31.6    mW = 31600 uW
    //   0 dBm =  1      mW =  1000 uW
    // -45 dBm = 0.00316 mW =  3.16 uW
    /*
     * 0.1 dB = 1,0233
     */
    char uW = (cBm < 0);
    float mW = 1;
    if (cBm > 0) for (int k=0; k<cBm; k++) mW = mW * 1.0233;
    else if (uW) for (int k=0; k>cBm; k--) mW = mW / 1.0233;
    if (uW) mW = mW*1000;
    int mWi = trunc(round(mW));
    sprintf(buffer, "%3.1u", mWi);
    if (uW) strcat(buffer, " uW");
    else    strcat(buffer, " mW"); 
    LCD_Pos(1,13);
    LCD_String(buffer, strlen(buffer)); 
}   //cBm2Lcd


//Anzeige der Frequenz in der Zeile 0
void Freq2Lcd(float freq_mhz, char line) {
    char str[21];
    for (char k=0; k<19; k++) str[k] = ' ';
    LCD_Pos(line,0);
  
    float fractpart, intpart;
    fractpart = modf(freq_mhz, &intpart);
    div_t output;
    output = div((long int) intpart, 1000);
    int ghz = output.quot;
    int mhz = output.rem;
   
    fractpart = fractpart*1000;
    fractpart = modf(fractpart, &intpart);
    
    int khz = intpart;
    fractpart = fractpart * 1000;
    int hz  = (int) fractpart;
    
    char buffer[5];
    sprintf(buffer, "%03.3u", ghz);   
    str[3] = buffer[2];

    sprintf(buffer, "%03.3u", mhz);   
    str[5] = buffer[0];    
    str[6] = buffer[1];    
    str[7] = buffer[2];   
    
    str[8] = '.';
    
    sprintf(buffer, "%03.3u", khz);   
    str[9]  = buffer[0];    
    str[10] = buffer[1];    
    str[11] = buffer[2];   
    
    sprintf(buffer, "%03.3u", hz);   
    str[13] = buffer[0];    
    str[14] = buffer[1];    
    str[15] = buffer[2];   
    
    str[17] = 'M';
    str[18] = 'H';
    str[19] = 'z';
    str[20] = 0;
    LCD_String(str, strlen(str)); 
}   //Freq2Lcd


// Anzeige einer integer zahl mit definierter lnge an definierter stelle
void Int2Lcd(int value, char length, char line, char pos){
    char str[10];
    char buffer[10];
    char diff;
    char bulen;
    for (char k=0; k<9; k++) str[k] = ' ';  
    sprintf(buffer, "%i", value);
    bulen = strlen(buffer);
    diff = length - bulen; 
    for (char k=diff; k<length; k++) str[k] = buffer[k-diff]; 
    
    LCD_Pos(line,pos);
    LCD_String(str, length); 
}

//Anzeige der Frequenz in der Zeile 'line'
void Freq2Lcd_I(signed long freq_Hz, char line) {
    char str[21];
    char buffer[10];
    for (char k=0; k<19; k++) str[k] = ' ';    
    
    sprintf(buffer, "%08.li", freq_Hz);    
  
    str[6] = buffer[0];    
    str[7] = buffer[1];    // MHz
    
    str[8] = '.';
    
    str[9]  = buffer[2];    
    str[10] = buffer[3];    
    str[11] = buffer[4];   //kHz
    
    str[13] = buffer[5];   
    str[14] = buffer[6];    
    str[15] = buffer[7];    // Hz
    
    str[17] = 'M';
    str[18] = 'H';
    str[19] = 'z';
    str[20] = 0;
    LCD_Pos(line,0);
    LCD_String(str, strlen(str)); 
}   //Freq2Lcd_I


//*****************************************************************************************************************
// EEPROM
//*****************************************************************************************************************

word ReadEepromWord(char adr){
    WORD buffer;
    buffer.byte0 = MyEeprom_read(adr);
    buffer.byte1 = MyEeprom_read(adr+1);
    return buffer._word;    
}  

void WriteEepromWord(char adr, word data){
    WORD buffer;
    buffer._word = data;
    MyEeprom_write(adr,   buffer.byte0);
    MyEeprom_write(adr+1, buffer.byte1);
} 

void WriteEepromFloat(char adr, float data){
    DWORD buffer;
    buffer._float = data;
    MyEeprom_write(adr,   buffer.byte0);
    MyEeprom_write(adr+1, buffer.byte1);
    MyEeprom_write(adr+2, buffer.byte2);
    MyEeprom_write(adr+3, buffer.byte3);
} 

float ReadEepromFloat(char adr){
    //Platzhalter
    DWORD buffer;
    buffer.byte0 = MyEeprom_read(adr);
    buffer.byte1 = MyEeprom_read(adr+1);
    buffer.byte2 = MyEeprom_read(adr+2);
    buffer.byte3 = MyEeprom_read(adr+3);
    return buffer._float;    
}    

void SaveFrqData(void){
    WriteEepromFloat(E_Adr_Quarz_Hz_I, Quarz_Hz_I);
} 

void LoadFrqData(void){
    Quarz_Hz_I = ReadEepromFloat(E_Adr_Quarz_Hz_I);
    
} 

void SaveCalData(void){
    char k;
    k = f2f();
    WORD buffer;
    buffer._int = PowerCal[k];
    MyEeprom_write(k*2,   buffer.byte0);
    MyEeprom_write(k*2+1, buffer.byte1);  
}

void SaveCal2Data(void){
    char k;
    k = f2f200();
    WORD buffer;
    buffer._int = PowerCal2[k];
    MyEeprom_write(E_Adr_PowerCal2 + k*2,   buffer.byte0);
    MyEeprom_write(E_Adr_PowerCal2 + k*2+1, buffer.byte1);  
}    

void ReadCalData(void){
    char k;
    WORD buffer;
    for (k=0;k<10;k++) {
        buffer.byte0 = MyEeprom_read(k*2);
        buffer.byte1 = MyEeprom_read(k*2+1);
        PowerCal[k]  = buffer._int;
    }
}

void ReadCal2Data(void){
    char k;
    WORD buffer;
    for (k=0;k<23;k++) {
        buffer.byte0 = MyEeprom_read(E_Adr_PowerCal2 + k*2);
        buffer.byte1 = MyEeprom_read(E_Adr_PowerCal2 + k*2+1);
        PowerCal2[k]  = buffer._int;
    }     
}

byte MyEeprom_read(byte addr) {
    EEADR            = addr;        // Address to read
    EECON1bits.EEPGD = 0;           // Point to Data Memory
    EECON1bits.CFGS  = 0;           // Access EEPROM
    EECON1bits.RD    = 1;           // EE Read
    return EEDATA;                  // W = EEDATA
    }

void MyEeprom_write(byte addr, byte n) {   

    EEADR            = addr;
    EEDATA           = n;
    di();
    EECON1bits.EEPGD = 0;             // Point to Data Memory
    EECON1bits.CFGS  = 0;             // Access EEPROM
    EECON1bits.WREN  = 1;

    INTCONbits.GIE   = 0;             // disattiva interrupt globali
    EECON2           = 0x55;          // Write 55h
    EECON2           = 0xAA;          // Write AAh
    EECON1bits.WR    = 1;             // abilita write.
    INTCONbits.GIE   = 1;             // attiva interrupt globali
    do {
        ClrWdt();
    } while(EECON1bits.WR);           // occupato ? 
    EECON1bits.WREN=0;                // disabilita write.
    ei();
  }