/*
 * File:   delays.c
 * Author: sprut
 *
 * Created on 4. Mrz 2020
 * 
 * alles was mit dem SPI zu tun hat
 * 
 * char / unsigned char
 * 
 */


#include "mytypes.h"
#include "signalgen.h"
#include "spi.h"


//*****************************************************************************************************************
// SPI
//*****************************************************************************************************************

/*
If more than 32 bits are written to the SPI interface, the most recent
32 bits, on the rising edge of LE, are clocked into the ADF435x.
The maximum SPI CLK speed is 20 MHz.
When AV DD is at 0 V, all SPI pins must be at 0 V.
 */

// send 32-bit data via SPI  MSB-first
// return with 255 if bus collission
//             0 if ok
char SPI32Tx(unsigned long int data) {
    unsigned long int data2;
    unsigned char nextbyte;
    data2 = data;
    di();
    char result = 0;
    unsigned char bytes[4]; 
    bytes[3] =  data2       & 0xFF;
    bytes[2] = (data2 >>8 ) & 0xFF;
    bytes[1] = (data2 >>16) & 0xFF;
    bytes[0] = (data2 >>24) & 0xFF;     
	for (char bytenummer=0; bytenummer<4; bytenummer++)
	{
    #ifdef SoftSPI
        SPI_CLK = 0;
        nextbyte = bytes[bytenummer]; 
        for (char bitnummer=0; bitnummer<8; bitnummer++) {
            if (nextbyte & 0x80) SPI_SDO = 1;
            else                 SPI_SDO = 0;
            __delay_us(SPI_Delay);
           SPI_CLK = 1; 
           __delay_us(SPI_Delay);
           SPI_CLK = 0;
           nextbyte = nextbyte << 1;            
        }    
    #else
        PIR1bits.SSPIF = 0;
        SSPBUF = bytes[bytenummer];         	// write byte to SSPBUF register
		if ( SSPCON1 & 0x80 ) result = 255;     // Fehler: Buskollision
		  else while( !PIR1bits.SSPIF );		// wait until bus cycle complete           
    #endif                    
	}  
    ei();
    return(result);
} //SPI32Tx


// send 8-bit data via SPI  LSB-first
// return with 255 if bus collission
//             0 if ok
char SPI8Tx(unsigned char data) {
    char result = 0;
    unsigned char mybyte;
    unsigned char nextbyte;
    mybyte = 0;
    if (data & 0x01) mybyte += 0x80;
    if (data & 0x02) mybyte += 0x40;
    if (data & 0x04) mybyte += 0x20;
    if (data & 0x08) mybyte += 0x10;
    if (data & 0x10) mybyte += 0x08;
    if (data & 0x20) mybyte += 0x04;
    if (data & 0x40) mybyte += 0x02;        
    di();  

    #ifdef SoftSPI
        SPI_CLK = 0;
        nextbyte = mybyte; 
        for (char bitnummer=0; bitnummer<8; bitnummer++) {
            if (nextbyte & 0x80) SPI_SDO = 1;
            else                 SPI_SDO = 0;
            __delay_us(SPI_Delay);
           SPI_CLK = 1; 
           __delay_us(SPI_Delay);
           SPI_CLK = 0;
           nextbyte = nextbyte << 1;            
        }  
    #else
    PIR1bits.SSPIF = 0;
    SSPBUF = mybyte;         	// write byte to SSPBUF register
    if ( SSPCON1 & 0x80 ) result = 255;     // Fehler: Buskollision
      else while( !PIR1bits.SSPIF );		// wait until bus cycle complete 	
    #endif
      
    ei();
    return(result);
}


/*errata:
 * In SPI mode, the SDO output may change after the
 * inactive clock edge of the bit ?0? output. This may
 * affect some SPI components that read data over
 * 300 ns after the inactive edge of SCK
 * 
 * clock has to be high enough to prevent halve-clocks below 300 ns -> 3,3 MHz 
 */

// master mode -> SS nicht gebraucht
void OpenSPI( unsigned char sync_mode, unsigned char bus_mode, unsigned char smp_phase) {
    #ifdef SoftSPI
    TRISBbits.TRISB1  = 0;
    TRISCbits.TRISC7  = 0;
    SPI_CLK = 0;
    #else
  SSPSTAT &= 0x3F;                // power on state 
  SSPCON1 =  0x00;                // power on state
  SSPCON1 |= sync_mode;           // select serial mode 
  SSPSTAT |= smp_phase;           // select data input sample phase

  switch( bus_mode ) {
    case 0:                       // SPI bus mode 0,0
      SSPSTATbits.CKE = 1;        // data transmitted on falling edge
      break;    
    case 2:                       // SPI bus mode 1,0
      SSPSTATbits.CKE = 1;        // data transmitted on rising edge
      SSPCON1bits.CKP = 1;        // clock idle state high
      break;
    case 3:                       // SPI bus mode 1,1
      SSPCON1bits.CKP = 1;        // clock idle state high
      break;
    default:                      // default SPI bus mode 0,1
      break;
  }
  switch( sync_mode ) {
    case 4:                       // slave mode w /SS enable
      TRISAbits.TRISA5 = 1;       // define /SS pin as input
	  TRISBbits.TRISB1 = 1;       // define clock pin as input
      break;
    case 5:                       // slave mode w/o /SS enable
	  TRISBbits.TRISB1 = 1;       // define clock pin as input
	  break;
	default:                      // master mode, define clock pin as output
	  TRISBbits.TRISB1 = 0;       // define clock pin as output
	 break;
  }
  TRISBbits.TRISB0 = 1;       // define SDI pin as input	
  TRISCbits.TRISC7 = 0;       // define SDO pin as output
//  SSPCON1 |= SSPEN;              // enable synchronous serial port   
  SSPCON1 |= SPI_SSPENB;         // enable synchronous serial port 
  
#endif
}  //OpenSPI  

//*****************************************************************************************************************


void OpenMyRs232(void){
    TRISCbits.TRISC6  = 0;
    RsTx = 0;
}


/* LSB first
 * Startbit:  1
 * Datenbits: 8
 * Stoppbit:  1
 * 
 */
void TxCharMyRs232(char value){   
    char k;
    //Startbit
    RsTx = 1;
    __delay_us(Rs_Delay);  // 104us = 9600
    //Datenbits
    for (k=0;k<8;k++) {
        if (value & 0x01) RsTx=0; else RsTx=1;
        value = value >> 1;
        __delay_us(Rs_Delay);
    }    
    //Stoppbit
    RsTx = 0;
    __delay_us(Rs_Delay);
}


void TxDWordMyRs232(unsigned long int data) {
    unsigned long int data2;    
    unsigned char bytes[4]; 
    data2 = data;
    bytes[3] =  data2       & 0xFF;
    bytes[2] = (data2 >>8 ) & 0xFF;
    bytes[1] = (data2 >>16) & 0xFF;
    bytes[0] = (data2 >>24) & 0xFF;     
    di();
	for (char bytenummer=0; bytenummer<4; bytenummer++) {
        TxCharMyRs232(bytes[bytenummer]);
    }
	ei();	
	};  
   